# -*- coding: utf-8 -*-

from PyQt4.QtCore import *
from PyQt4.QtGui import QImage, QPainter, QWidget, QLabel, QHBoxLayout, QColor, QCursor
from PyKDE4.plasma import Plasma
from PyKDE4.kdeui import KApplication, KWindowSystem, KColorButton
from PyKDE4.kdecore import i18n
from PyKDE4 import plasmascript
import random

NekoWalkArray = 'neko-walk01.svg', 'neko-walk02.svg', 'neko-walk03.svg', 'neko-walk04.svg', 'neko-walk05.svg', 'neko-walk06.svg', 'neko-walk07.svg', 'neko-walk08.svg', 'neko-walk09.svg', 'neko-walk10.svg'

ANIMATIONSPEED = 25

class BNeko(plasmascript.Applet):
    def __init__(self,parent,args=None):
        plasmascript.Applet.__init__(self,parent)

    def init(self):
        self.setAspectRatioMode(Plasma.KeepAspectRatio)

        self.setHasConfigurationInterface(False)
        self.setBackgroundHints(Plasma.Applet.NoBackground)

        self.animationTimePassed = 0
        self.animationChange = 0

        self.isMirrored = True
        self.catColor = Qt.black

        self.svgArray = []
        for x in range(10):
            svg = Plasma.Svg(self)
            svg.setImagePath(self.package().filePath("images", NekoWalkArray[x]))
            self.svgArray.append(svg)

        self.indexToPaint = 0

        self.screenSize = self.screenSize()
        self.setMaximumSize(QSizeF(self.screenSize.width() / 5.0, self.screenSize.width() / 5.0 * self.aspectRatio()))
        self.resize(QSizeF(self.maximumWidth()/2.0, self.maximumHeight()/2.0))
        self.updateSvgSize(self.size())

        self.setHasConfigurationInterface(True);
        self.readConfiguration()

        self.mousePressed = False

        self.timer = QTimer(self)
        self.connect(self.timer, SIGNAL("timeout ()"), self.animation)
        self.timer.start(ANIMATIONSPEED)

    def screenSize(self):
        app = KApplication.kApplication()
        if app.desktop().screenCount() > 1 or app.desktop().isVirtualDesktop():
            screenNum = KApplication.desktop().screenNumber(QCursor.pos()) #unreliable
            return KWindowSystem.workArea().intersect(app.desktop().screenGeometry(screenNum))
        else:
            return KWindowSystem.workArea()

    def aspectRatio(self):
        return (360.0/450.0)
 
    def paintInterface(self, painter, option, rect):
        painter.setRenderHints(QPainter.SmoothPixmapTransform)

        image = QImage(rect.width(), rect.height(), QImage.Format_ARGB32)
        painterImage = QPainter(image)
        painterImage.setCompositionMode(QPainter.CompositionMode_Source)
        painterImage.fillRect(image.rect(), Qt.transparent)
        painterImage.setCompositionMode(QPainter.CompositionMode_SourceOver)
        self.svgArray[self.indexToPaint].paint(painterImage, 0, 0)
        painterImage.setCompositionMode(QPainter.CompositionMode_SourceIn)
        painterImage.fillRect(image.rect(), self.catColor)
        painterImage.end()
        if self.isMirrored == True:
            image = image.mirrored(True, False)
        painter.drawImage(0,0, image)

    def updateSvgSize(self, size):
        for x in range(10):
            self.svgArray[x].resize(size)

    def animation(self):
        if self.mousePressed:
            return

        moveSpeed = 4

        newX = self.x()
        newY = self.y()

        if self.isMirrored == False:
            newX -= moveSpeed
            if newX + self.geometry().width() < self.screenSize.y():
                self.isMirrored = True
                newY = random.random() * (self.screenSize.height() - self.geometry().height())
                #print newY
        else:
            newX += moveSpeed
            if newX > self.screenSize.width():
                self.isMirrored = False
                newY = random.random() * (self.screenSize.height() - self.geometry().height())
                #print newY

        #print newX
        self.setPos(newX, newY)

        if self.animationChange == 2:
            self.indexToPaint += 1
            if self.indexToPaint == 10:
                self.indexToPaint = 0
            self.animationChange = 0
            self.update()
        else:
            self.animationChange += 1

    def constraintsEvent(self, constraints):
        if constraints & Plasma.FormFactorConstraint:
            self.setBackgroundHints(Plasma.Applet.NoBackground)

        if constraints & Plasma.SizeConstraint:
            self.updateSvgSize(self.size())

    def mouseReleaseEvent(self, event):
        #print("release")
        self.mousePressed = False
        event.accept()

    def mousePressEvent(self, event):
        #print("press")
        self.mousePressed = True
        event.accept()

    def createConfigurationInterface(self, parent):
        self.widget = QWidget()
        label = QLabel(i18n("Color"), self.widget)
        self.colorBtn = KColorButton()
        self.colorBtn.setColor(self.catColor)

        layout = QHBoxLayout()
        layout.addWidget(label)
        layout.addWidget(self.colorBtn)

        self.widget.setLayout(layout)

        parent.addPage(self.widget, i18n("General"), self.icon())
        self.connect(parent, SIGNAL("accepted ()"), self.configAccepted)
        self.connect(parent, SIGNAL("rejected ()"), self.configRejected)

    def configAccepted(self):
        self.mousePressed = False
        self.catColor = self.colorBtn.color()
        cg = self.config();
        cg.writeEntry("Color", self.catColor.name())
        self.configNeedsSaving();
        self.update()

    def configRejected(self):
        self.mousePressed = False

    def readConfiguration(self):
        cg = self.config();
        self.catColor = QColor(cg.readEntry("Color", "#000000"))


def CreateApplet(parent):
    return BNeko(parent)
